package com.cosmic_dust.flutter_hyphenate

import android.content.ContentValues.TAG
import android.content.Context
import android.os.Handler
import android.util.Log
import com.hyphenate.EMCallBack
import com.hyphenate.EMMessageListener
import com.hyphenate.chat.EMClient
import com.hyphenate.chat.EMMessage
import com.hyphenate.chat.EMOptions
import com.hyphenate.chat.EMTextMessageBody
import com.hyphenate.exceptions.HyphenateException
import io.flutter.plugin.common.EventChannel.EventSink
import io.flutter.plugin.common.MethodChannel.Result


class HyphenateClient {

    fun initHyphenate(context: Context, result: Result, appKey: String, isDebug: Boolean = true, handler: Handler) {
        try {
            val options = EMOptions()
            options.appKey = appKey
            options.acceptInvitationAlways = false
            EMClient.getInstance().init(context, options)
            EMClient.getInstance().setDebugMode(isDebug)
            handler.post { result.success("init success") }
        } catch (e: Exception) {
            Log.e(TAG, "init fail", e)
            result.error("init fail", null, e)
        }
    }

    fun createAccount(username: String?, password: String?, result: Result, handler: Handler) {
        try {
            EMClient.getInstance().createAccount(username, password)
            handler.post { result.success("create success") }
        } catch (e: HyphenateException) {
            Log.e(TAG, "create fail", e)
            result.error("create fail", "errorCode=" + e.errorCode, e)
        }
    }

    fun login(username: String?, password: String?, result: Result, handler: Handler) {
        EMClient.getInstance().login(username, password, object : EMCallBack {
            //回调
            override fun onSuccess() {
                handler.post {
                    EMClient.getInstance().groupManager().loadAllGroups()
                    EMClient.getInstance().chatManager().loadAllConversations()
                    result.success("login success")
                }
                Log.i("main", "login success!")
            }

            override fun onProgress(progress: Int, status: String) {}
            override fun onError(code: Int, message: String) {
                handler.post { result.error("login fail", "$message\ncode:$code", null) }
                Log.i("main", "login fail:" + message + "code" + code)
            }
        })
    }

    fun logout(result: Result, handler: Handler) {
        EMClient.getInstance().logout(true, object : EMCallBack {
            override fun onSuccess() {
                handler.post { result.success("logout success") }
                Log.i("main", "logout success!")
            }

            override fun onProgress(progress: Int, status: String) {}
            override fun onError(code: Int, message: String) {
                handler.post { result.error("logout fail", "$message\ncode:$code", null) }

                Log.i("main", "logout fail:" + message + "code" + code)
            }
        })
    }

    fun sendTxtMsg(toChatUsername: String?, content: String?, isGroupChat: Boolean, result: Result, handler: Handler) {
        try {
            val message = EMMessage.createTxtSendMessage(content, toChatUsername)
            if (isGroupChat) {
                message.chatType = EMMessage.ChatType.GroupChat
            }
            EMClient.getInstance().chatManager().sendMessage(message)
            handler.post { result.success("sendTxtMsg success") }

        } catch (e: java.lang.Exception) {
            Log.e(TAG, "sendTxtMsg fail", e)
            handler.post { result.error("sendTxtMsg fail", null, e) }

        }
    }

    fun addEMMessageListener(eventSink: EventSink, handler: Handler) {
        val msgListener: EMMessageListener = object : EMMessageListener {
            override fun onMessageReceived(messages: List<EMMessage>) {
                for (message in messages) {
                    if (message.type == EMMessage.Type.TXT) {
                        val body = message.body as EMTextMessageBody
                        val event: MutableMap<String, String> = HashMap()
                        event["eventType"] = "onMessageReceived"
                        event["message"] = body.message
                        event["type"] = "txt"
                        handler.post { eventSink.success(event) }
                    }
                }
            }

            override fun onCmdMessageReceived(messages: List<EMMessage>) {}
            override fun onMessageRead(messages: List<EMMessage>) {}
            override fun onMessageDelivered(messages: List<EMMessage>) {}
            override fun onMessageRecalled(messages: List<EMMessage>) {}
            override fun onMessageChanged(message: EMMessage, change: Any) {}
        }
        EMClient.getInstance().chatManager().addMessageListener(msgListener)
    }

}